<?xml version="1.0" encoding="UTF-8"?>
<sch:schema xmlns:sch="http://purl.oclc.org/dsdl/schematron" queryBinding="xslt2"
    xmlns:sqf="http://www.schematron-quickfix.com/validator/process" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    
    <sch:ns uri="http://www.oxygenxml.com/customFunction" prefix="oxyF"/>
    <sch:ns uri="http://www.oxygenxml.com/ai/function" prefix="ai"/>
    <sch:ns uri="java:ro.sync.ecss.dita.extensions.DITAXSLTExtensionFunctionUtil" prefix="ditaaccess"/>
    
    <!-- Ensures the root element contains a prolog with author and critical dates. Provides a quick
        fix to add the missing prolog if it is absent. -->
    <sch:pattern>
        <sch:rule context="/*">
            <sch:assert test="prolog" sqf:fix="addProlog">
                Should contain prolog with the author name and critical dates.
            </sch:assert>
            
            
            <!-- Quick fix that adds the missing cells from a table. -->
            <sqf:fix id="addProlog">
                <sqf:description>
                    <sqf:title>[AI] Add prolog</sqf:title>
                    <sqf:p>Add a prolog to the current topic. Genearte the index term using AI</sqf:p>
                </sqf:description>
                <sqf:add match="(title | titlealts | abstract | shortdesc)[last()]" position="after">
                    <prolog>
                        <author>
                        </author>
                        <critdates>
                            <created>
                                <xsl:attribute name="date">
                                    <xsl:value-of select="format-dateTime(current-dateTime(),'[Y]-[M01]-[D01]')"/>
                                </xsl:attribute>
                            </created>
                        </critdates>
                        <xsl:call-template name="generate-keywords">
                            <xsl:with-param name="doc" select="parent::node()" />
                        </xsl:call-template>
                    </prolog>
                </sqf:add>
            </sqf:fix>
        </sch:rule>
    </sch:pattern>
    
    <!-- Generates a DITA XML keywords element from the most used words in the provided document.
        Utilizes an AI transformation to extract up to five index terms from the text content. -->
    <xsl:template name="generate-keywords">
        <xsl:param name="doc" />
        <metadata>
            <xsl:value-of disable-output-escaping="yes" select="ai:transform-content('Act as a technical writer assistant. 
                Create a DITA XML keywords element containing a maximum of 5 indexterm elements 
                extracted from the most used words in the following text content.\nAnswer without 
                other explanations returning directly the keywords element without any prefix text. Do not add markdown markers.', $doc)" />
        </metadata>
    </xsl:template>
    
    <!-- Recommends adding an 'indexterm' element within the root element if missing. Provides an AI
        -generated fix to automatically insert the 'indexterm' using a specified template. -->
    <sch:pattern>
        <sch:rule context="prolog">
            <sch:assert test="metadata/keywords/indexterm" role="warn" sqf:fix="addFragment">
                It is recommended to add an 'indexterm' in the current '<sch:name/>' element.
            </sch:assert>
            
            <sqf:fix id="addFragment">
                <sqf:description>
                    <sqf:title>[AI] Add the 'indexterm' element</sqf:title>
                    <sqf:p>Generate the index terms from the document using AI</sqf:p>
                </sqf:description>
                <sch:let name="doc" value="."/>
                <!-- Add the indexterm element element and its parents -->
                <sqf:add position="last-child">
                    <xsl:call-template name="generate-keywords">
                        <xsl:with-param name="doc" select="parent::node()" />
                    </xsl:call-template>
                </sqf:add>
            </sqf:fix>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern>
        <sch:rule context="*[local-name() = ('topic', 'concept', 'task', 'reference')][title]">
            <sch:assert test="shortdesc" role="warn" sqf:fix="addShortdesc">
                It is recommended to add an 'shortdesc' in the current.
            </sch:assert>
            
            <sqf:fix id="addShortdesc">
                <sqf:description>
                    <sqf:title>[AI] Add a short description for the current document</sqf:title>
                    <sqf:p>Generate a short description for the current document using AI</sqf:p>
                </sqf:description>
                <!-- Add the indexterm element element and its parents -->
                <sqf:add match="title" position="after">
                    <shortdesc>
                        <xsl:value-of select="
                            ai:transform-content('You are a technical documentation writer. Generate a short description as text 
                            in less than 30 words for this content:',
                            string-join(parent::*//text(), ''))"/>
                    </shortdesc>
                </sqf:add>
            </sqf:fix>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern id="no_alt_desc">
        <!-- EXM-28035 Avoid reporting warnings when the image has a @conref or @conkeyref, the attribute might be on the target. -->
        <sch:rule context="*[contains(@class, ' topic/image ')][not(@conref)][not(@conkeyref)]">
            <sch:assert test="@alt | alt" 
                flag="non-WAI" role="warning" sqf:fix="addAltElem" see="https://www.oxygenxml.com/dita/1.3/specs/langRef/base/image.html">                
                Images must have text alternatives that describe the information or function represented by them.</sch:assert>
            
            <sqf:fix id="addAltElem">
                <sqf:description>
                    <sqf:title>[AI] Add alt element</sqf:title>
                    <sqf:p>Generate the alternate image text by reading the image with AI</sqf:p>
                </sqf:description>
                <sqf:add node-type="element" target="alt">
                    <xsl:value-of select="
                        ai:transform-content(
                        'Create a short alternate text description for this image:',
                        concat('${attach(', resolve-uri(@href, base-uri()), ')}'))"/>
                </sqf:add>                
            </sqf:fix>
        </sch:rule>
    </sch:pattern>   
    
    <sch:pattern>
        <sch:rule context="/*[1][contains(@class, ' topic/topic ')]">
            <sch:let name="reqId" value="replace(tokenize(document-uri(/), '/')[last()], '\.dita', '')"/>
            <sch:assert test="@id = $reqId" sqf:fix="setId"> Topic ID must be equal to file name. </sch:assert>
            <sqf:fix id="setId">
                <sqf:description>
                    <sqf:title>Set "<sch:value-of select="$reqId"/>" as a topic ID</sqf:title>
                    <sqf:p>The topic ID must be equal to the file name.</sqf:p>
                </sqf:description>
                <sqf:add node-type="attribute" target="id" select="$reqId"/>
            </sqf:fix>
        </sch:rule>
    </sch:pattern>
    
    
    <sch:pattern>
        <sch:rule context="*[contains(@class, ' topic/xref ') or contains(@class, ' topic/link ')]">
            <sch:report test="@scope = 'external' and @href = text()" sqf:fix="removeText"> Link text
                is same as @href attribute value. Please remove. </sch:report>
            <sqf:fix id="removeText">
                <sqf:description>
                    <sqf:title>Remove redundant link text, text is same as @href value.</sqf:title>
                </sqf:description>
                <sqf:delete match="text()"/>
            </sqf:fix>
            <!-- Check that the scope attribute is set. -->
            <sch:report test="@format = 'html' and (not(@scope) or @scope != 'external')"
                sqf:fix="setScopeAttr" role="warn"> External link should have a scope attribute set to
                external </sch:report>
            <sqf:fix id="setScopeAttr">
                <sqf:description>
                    <sqf:title>Set scope attribute to external</sqf:title>
                </sqf:description>
                <sqf:add node-type="attribute" target="scope" select="'external'"/>
            </sqf:fix>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern>
        <!-- Image without any kind of reference -->
        <sch:rule context="*[contains(@class, ' topic/image ')]">
            <sch:report test="not(@href) and not(@keyref) and not(@conref) and not(@conkeyref)"
                sqf:fix="add_href add_keyref add_conref add_conkeyref"> Image without a reference. </sch:report>
            <sqf:fix id="add_href">
                <sqf:description>
                    <sqf:title>Add @href attribute</sqf:title>
                </sqf:description>
                <sqf:add node-type="attribute" target="href"/>
            </sqf:fix>
            <sqf:fix id="add_keyref">
                <sqf:description>
                    <sqf:title>Add @keyref attribute</sqf:title>
                </sqf:description>
                <sqf:add node-type="attribute" target="keyref"/>
            </sqf:fix>
            <sqf:fix id="add_conref">
                <sqf:description>
                    <sqf:title>Add @conref attribute</sqf:title>
                </sqf:description>
                <sqf:add node-type="attribute" target="conref"/>
            </sqf:fix>
            <sqf:fix id="add_conkeyref">
                <sqf:description>
                    <sqf:title>Add @conkeyref attribute</sqf:title>
                </sqf:description>
                <sqf:add node-type="attribute" target="conkeyref"/>
            </sqf:fix>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern>
        <!-- Report cases when the lines in a codeblock exceeds 95 characters -->
        <sch:rule
            context="*[contains(@class, ' pr-d/codeblock ') and not(@outputclass = 'language-css') and not(@outputclass = 'language-java') and not(@outputclass = 'language-bourne')]"
            role="warn">
            <sch:let name="offendingLines" value="oxyF:lineLengthCheck(string(), 150)"/>
            <sch:report test="string-length($offendingLines) &gt; 0"> Lines (<sch:value-of
                    select="$offendingLines"/>) in codeblocks should not exceed 150 characters.
            </sch:report>
        </sch:rule>
    </sch:pattern>
    
    <sch:pattern>
        <!-- Report possible case in which a codeblock containg XML was not marked appropriately. -->
        <sch:rule context="*[contains(@class, ' pr-d/codeblock ')]" role="warn">
            <sch:report test="not(@outputclass)" sqf:fix="add_outputclass"> Possible XML Codeblock
                without @outputclass set to it. </sch:report>
            <sqf:fix id="add_outputclass">
                <sqf:description>
                    <sqf:title>[AI] Add @outputclass attribute to codeblock</sqf:title>
                </sqf:description>
                <sqf:add node-type="attribute" target="outputclass">
                    <xsl:value-of select="
                        ai:transform-content('Determine the value for Dita outputclass attribute for styling the following content. 
                        Return just the attribute value without explanations.',.)"/>
                </sqf:add>
            </sqf:fix>
        </sch:rule>
    </sch:pattern>
    
    <xsl:function name="oxyF:lineLengthCheck" as="xs:string">
        <xsl:param name="textToBeChecked" as="xs:string"/>
        <xsl:param name="maxLength" as="xs:integer"/>
        <xsl:value-of>
            <xsl:for-each select="tokenize($textToBeChecked, '\n')">
                <xsl:if test="string-length(current()) &gt; $maxLength">
                    <xsl:value-of select="concat(position(), ' - ', string-length(current()), ', ')"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:value-of>
    </xsl:function>
</sch:schema>